local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher" 
local wa = require "luci.tools.webadmin"
local sys = require "luci.sys"
local fs = require "nixio.fs"

local query_id
if arg[1] then
    query_id = arg[1]
end

m = Map("modbus-master", translate("Query Register Mapping"))

local cfgName
m.uci:foreach("modbus-master", "modbus_query", function(s)
	if s.query_id ==  query_id then
		cfgName = s[".name"]
	end
end)

s = m:section(NamedSection, cfgName, "Query", "Modbus Query")
--s.template = "modbus_master/tblsection"

function s:filter(section)
	local s_query_id = self.map.uci:get(self.config, section, "query_id")
	if(s_query_id == query_id) then
		return section
	else
		return nil
	end
end

name = s:option(Value, "name", translate("Query Name"))
name.optional = false

f_code = s:option(ListValue, "function", translate("Modbus Function"))
f_code:value("1", translate("Read coils (1)"))
f_code:value("2", translate("Read input coils (2)"))
f_code:value("3", translate("Read holding registers (3)"))
f_code:value("4", translate("Read input registers (4)"))
f_code.default = "3"

start_address = s:option(Value, "start_address", translate("Start Address"))
start_address.datatype = "range(1,65536)"
start_address.default = "1"
start_address.rmempty = false

reg_count = s:option(Value, "reg_count", translate("Count"))
reg_count.default = "1"
reg_count.rmempty = false
reg_count.validator_hint = translate("Single or multiple integers allowed.")

slave_id = s:option(Value, "slave_id", translate("Slave Id"))
slave_id.default = "1"
slave_id.datatype = "range(1,255)"

polling_interval = s:option(Value, "polling_interval", translate("Polling Interval (Seconds)"))
polling_interval.datatype = "range(1,65536)"
polling_interval.default = "60"
polling_interval.rmempty = false

enabled = s:option(Flag, "enabled", translate("Enabled"))
enabled.default = true


s = m:section(TypedSection, "modbus_register", translate("Modbus Registers"))
s.template = "cbi/tblsection"
s.addremove = true
s.anonymous = true
s.add_toggled = true
s.novaluetext = translate("No registers configured for this slave device yet")

function s:filter(section)
	local s_query_id = self.map.uci:get(self.config, section, "query_id")
	if(s_query_id == query_id) then
		return section
	else
		return nil
	end
end

function s.create(self, section, ...)
	local register_id = sys.exec("head -c 32 /dev/urandom | md5sum | awk '{ print $1 }' | tr -d '\n'")
	local created = TypedSection.create(self, section, ...)
	self.map:set(created, "register_id", register_id)
	self.map:set(created, "query_id", query_id)

	if created then
		luci.http.redirect(ds.build_url("admin/services/modbus/query_mapping", query_id))
	end
end

address = s:option(Value, "address", translate("Address"), translate(""))
address.datatype = "integer"

name = s:option(Value, "name", translate("Name"), translate(""))
name.datatype = "string"

data_type = s:option(ListValue, "data_type", translate("Dataype"))
data_type:value("bit", translate("Bit (Coils/Inputs)"))
data_type:value("int", translate("Integer"))
data_type:value("float_abcd", translate("32bit float, Byte order 1,2,3,4"))
data_type:value("float_dcba", translate("32bit float, Byte order 4,3,2,1"))
data_type:value("float_badc", translate("32bit float, Byte order 2,1,4,3"))
data_type:value("float_cdab", translate("32bit float, Byte order 3,4,1,2"))
data_type.default = "none"
data_type.rmempty = false

scaling_a = s:option(Value, "scaling_a", translate("Scaling A"), translate(""))
scaling_a.datatype = "float"
scaling_a.default = 1

scaling_b = s:option(Value, "scaling_b", translate("Scaling B"), translate(""))
scaling_b.datatype = "float"
scaling_b.default = 0

-- process_or_config = s:option(ListValue, "process_or_config", translate("Process Or Config"))
-- process_or_config:value("process", translate("Process"))
-- process_or_config:value("config", translate("Config"))
-- process_or_config.default = "none"
-- process_or_config.rmempty = false

hi_threashold = s:option(Value, "hi_threashold", translate("High Threashold"), translate(""))
hi_threashold.datatype = "range(0,10000)"

low_threashold = s:option(Value, "low_threashold", translate("Low Threashold"), translate(""))
low_threashold.datatype = "range(0,10000)"

send_alarm = s:option(ListValue, "send_alarm", translate("Send Alarm"))
send_alarm:value("high", translate("High"))
send_alarm:value("low", translate("Low"))
send_alarm:value("none", translate("None"))
send_alarm.default = "none"
send_alarm.rmempty = false

return m
