local uci = require "luci.model.uci".cursor()
local ds = require "luci.dispatcher"
local wa = require "luci.tools.webadmin"
local sys = require "luci.sys"
 
local connection_id
if arg[1] then
	connection_id = arg[1]
else
	luci.http.redirect(ds.build_url("admin/services/modbus/modbus_master"))
end

local modbus_query_section = "modbus_query"
local alert_list = "modbus_register"

m = Map("modbus-master")
m.redirect = ds.build_url("admin/services/modbus/modbus_master")

local cfgName
m.uci:foreach("modbus-master", "modbus_device", function(s)
	if s.connection_id == connection_id then
		cfgName = s[".name"]
	end
end)

s = m:section(NamedSection, cfgName, "Slave device configuration", "Slave device configuration")
s.add_toggled = true

enabled = s:option(Flag, "enabled", translate("Enabled"), translate("Check to enable this Modbus slave configuration"))


conf_name = s:option(Value, "conf_name", translate("Device Name"))
conf_name.datatype = "rangelength(1, 32)"

modbus_type = s:option(ListValue, "type", translate("Type"))
modbus_type:value("rtu", translate("Modbus RTU"))
modbus_type:value("tcp", translate("Modbus TCP"))
modbus_type.default = "none"

host = s:option(Value, "host", translate("IP address"))
host.datatype = "ip4addr"
host:depends("type", "tcp")

port = s:option(Value, "port", translate("Port"))
port.datatype = "port"
port.default = "502"
port:depends("type", "tcp")

timeout = s:option(Value, "timeout", translate("Timeout (seconds)"))
timeout.datatype = "range(1,30)"
timeout.default = "5"

serial_port = s:option(Value, "serial_port", translate("Serial Port"))
serial_port.datatype = "string"
serial_port.default = "/dev/ttyS0"
serial_port:depends("type", "rtu")

baudrate = s:option(ListValue, "baudrate", translate("Baud rate"))
baudrate:value("300", "300")
baudrate:value("1200", "1200")
baudrate:value("2400", "2400")
baudrate:value("4800", "4800")
baudrate:value("9600", "9600")
baudrate:value("19200", "19200")
baudrate:value("38400", "38400")
baudrate:value("57600", "57600")
baudrate:value("115200", "115200")
baudrate.default = "115200"
baudrate:depends("type", "rtu")

parity = s:option(ListValue, "parity", translate("Parity"))
parity:value("N", translate("None"))
parity:value("E", translate("Even"))
parity:value("O", translate("Odd"))
parity.default = "N"
parity:depends("type", "rtu")

databits = s:option(ListValue, "databits", translate("Data bits"))
databits:value("5", "5")
databits:value("6", "6")
databits:value("7", "7")
databits:value("8", "8")
databits.default = "8"
databits:depends("type", "rtu")

stopbits = s:option(ListValue, "stopbits", translate("Stop bits"))
stopbits:value("1", "1")
stopbits:value("2", "2")
stopbits.default = "1"
stopbits:depends("type", "rtu")

-- mode = s:option(ListValue, "mode", translate("Mode"))
-- mode:value("RS232", translate("RS232"))
-- mode:value("RS485", translate("RS485"))
-- mode.default = "none"
-- mode:depends("type", "rtu")

-- rts = s:option(ListValue, "rts", translate("RTS"))
-- rts:value("up", translate("Up"))
-- rts:value("down", translate("Down"))
-- rts:value("none", translate("None"))
-- rts.default = "none"
-- rts:depends("type", "rtu")

-- rts_delay = s:option(Value, "rts_delay", translate("RTS Delay"))
-- rts_delay.datatype = "range(0, 32)"
-- rts_delay.default = "1"
-- rts_delay:depends("type", "rtu")

byte_timeout = s:option(Value, "byte_timeout", translate("Byte Timeout"))
byte_timeout.datatype = "range(0, 32)"
byte_timeout.default = "8"
byte_timeout:depends("type", "rtu")

s = m:section(TypedSection, modbus_query_section, translate("Modbus Queries"))
s.template = "modbus_master/tblsection"
s.map_button = true
s.addremove = true
s.anonymous = true
s.add_toggled = true
s.novaluetext = translate("No requests configured for this slave device yet")

function s:filter(section)
	local connection = self.map.uci:get(self.config, section, "connection")
	if(connection == connection_id) then
		return section
	else
		return nil
	end
end

function s.create(self, section, ...)
	local query_id = sys.exec("head -c 32 /dev/urandom | md5sum | awk '{ print $1 }' | tr -d '\n'")
	local created = TypedSection.create(self, section, ...)
	self.map:set(created, "query_id", query_id)
	self.map:set(created, "connection", connection_id)

	if created then
		luci.http.redirect(ds.build_url("admin/services/modbus/modbus_master", connection_id))
	end
end

function s.remove(self, section)
	stat = TypedSection.remove(self, section)
	return stat
end

function s.map_button(self, section)
	local query_id = self.map.uci:get("modbus-master", section, "query_id")
	return ds.build_url("admin", "services", "modbus", "query_mapping", query_id)
end

name = s:option(Value, "name", translate("Query Name"))
name.optional = false

f_code = s:option(ListValue, "function", translate("Modbus Function"))
f_code:value("1", translate("Read coils (1)"))
f_code:value("2", translate("Read input coils (2)"))
f_code:value("3", translate("Read holding registers (3)"))
f_code:value("4", translate("Read input registers (4)"))
f_code.default = "3"

start_address = s:option(Value, "start_address", translate("Start Address"))
start_address.datatype = "range(1,65536)"
start_address.default = "1"
start_address.rmempty = false

reg_count = s:option(Value, "reg_count", translate("Count"))
reg_count.default = "1"
reg_count.rmempty = false
reg_count.validator_hint = translate("Single or multiple integers allowed.")

slave_id = s:option(Value, "slave_id", translate("Slave Id"))
slave_id.default = "1"
slave_id.datatype = "range(1,255)"

polling_interval = s:option(Value, "polling_interval", translate("Polling Interval (Seconds)"))
polling_interval.datatype = "range(1,65536)"
polling_interval.default = "60"
polling_interval.rmempty = false

enabled = s:option(Flag, "enabled", translate("Enabled"))
enabled.default = true

return m
